/********** Version 1  3/18/87  8:26:25
* Copyright (c) 1987 by Phoenix Systems, Inc.
* Filename: wpc.c
* Author..: Robert V. Hannah III
* Date....:  3/18/87
*
* C routines for WordPerfect emulation in Clipper memo fields, and routines
* to print those memos without the soft carriage return.
*
**********/
#include <extend.h>
#include "wpc.h"

/**************
* Set initial screen coordinates and determine video mode.
**************/
void cwpsetrc()
{
   tr = _parni( 1 );
   lc = _parni( 2 );
   br = _parni( 3 );
   rc = _parni( 4 );

   fptr = (char *) VIDMODE;   /* Get video byte... */

   switch ( *fptr )           /* ...and set our screen memory pointer accordingly. */
   {
      case 0:
      case 1:
      case 2:
      case 3:
         fptr = (char *) COLOMEM;
         break;
      case 7:
         fptr = (char *) MONOMEM;
         break;
      default:
         fptr = (char *) NULL;
         break;
   }
   _ret();
}

/**************
* This routine emulates "delete to end of page" by returning a character string
* which is stuffed in the keyboard.  As there is no "page", it deletes to the
* end of the screen.
**************/
void cwpcpgdn()
{
   char *ptr;
   int x;
   char ret[110], *tmp;
   char *scr = _parc( 1 );          /* the screen image, plus attributes */
   int r     = _parni( 2 );
   int c     = _parni( 3 );
   int flag  = 0;
   int line;

   /* Find a non-blank line */

   for ( x = br; x > r && ! flag; x-- )
   {
      tmp = scr + x * 160 + rc * 2;
      for ( ptr = scr + x * 160 + lc * 2; ptr < tmp && ! flag; ptr += 2 )
         if ( *ptr > ' ' && *ptr <= '~' )
            flag++;
   }

   line = flag ? ++x : 0;                 /* If flag = 0 don't delete lines */
   flag = (rc - c + 1) / 2;               /* Max # of words possible */
   for ( x = 0, ptr = ret; x < flag; x++ )   /* Delete every word */
      *ptr++ = 20;                        /* Del */

   *ptr++ = 24;                           /* Down */
   for ( x = line; x > r; x-- )           /* Delete number of lines found */
      *ptr++ = 25;                        /* ^Y - Delete line */

   *ptr++ = 5;                            /* Up */
   for ( x = lc; x < c; x++ )             /* Number of characters to move */
      *ptr++ = 4;                         /* Right */

   *ptr = 0;                              /* Null terminate line... */
   _retc( ret );                          /* ...and return buffer */
}

/*************
* Prints a screen of help.  Access video memory directly to speed things up
* and to avoid having to twiddle Clipper's thoughts on color.
*************/
void cwphelp()
{
   char *myptr, *ptr;
   int page = _parni( 1 );
   int line;      /* Absolute line is page * 11 + line -- line is 0 to 10 */
   int x, y;

   line = helppage[ page ].top_line;

   if ( page == 0 )
   {
      /* Draw box - reverse video */

      for ( y = HLP_LFT; y <= HLP_RGT; y++ )          /* Top and bottom */
      {
         *(fptr + ((HLP_TOP * 80 + y) * 2))     = SL_HOR;
         *(fptr + ((HLP_TOP * 80 + y) * 2) + 1) = REVVID;
         *(fptr + ((HLP_BOT * 80 + y) * 2))     = SL_HOR;
         *(fptr + ((HLP_BOT * 80 + y) * 2) + 1) = REVVID;
      }

      for ( x = HLP_TOP; x <= HLP_BOT; x++ )          /* Left and right */
      {
         *(fptr + ((x * 80 + HLP_LFT) * 2))     = SL_VER;
         *(fptr + ((x * 80 + HLP_LFT) * 2) + 1) = REVVID;
         *(fptr + ((x * 80 + HLP_RGT) * 2))     = SL_VER;
         *(fptr + ((x * 80 + HLP_RGT) * 2) + 1) = REVVID;
      }

      *(fptr + (HLP_TOP * 80 + HLP_LFT) * 2) = SL_UL; /* The corners */
      *(fptr + (HLP_TOP * 80 + HLP_RGT) * 2) = SL_UR;
      *(fptr + (HLP_BOT * 80 + HLP_LFT) * 2) = SL_LL;
      *(fptr + (HLP_BOT * 80 + HLP_RGT) * 2) = SL_LR;

      y = (HLP_RGT - HLP_LFT - strlen( helpcmd )) / 2 + HLP_LFT;
      for ( ptr = helpcmd; *ptr; y++ )
      {
         *(fptr + ((HLP_BOT * 80 + y) * 2))     = *ptr++;
         *(fptr + ((HLP_BOT * 80 + y) * 2) + 1) = REVVID;
      }
   }

   /* Draw all text for the current page */

   for ( x = HLP_TOP + 1; x < HLP_BOT; x++, line++ )
   {
      if (line > helppage[ page ].bot_line)
         ptr = helpblnk;
      else
         ptr = help[ line ];

      y     = HLP_LFT + 1;
      myptr = fptr + ((x * 80 + y) * 2);
      for ( ; y < HLP_RGT; y++ )
      {
         *myptr++ = *ptr++;
         *myptr++ = NORMVID;
      }
   }

   _ret();
}

/************
* Gets rid of all soft carraige returns.
************/
void cwphcr()
{
   unsigned char *pt1, *pt2, *ret;

   for ( pt1 = pt2 = ret = _parc( 1 ); *pt2; pt2++ )
   {
      if ( *pt2 == 0x8d || (*pt2 == 0x0a && *(pt2 - 1) == 0x8d) )
         continue;

      if ( *pt2 == 0x0d && *(pt2 + 1) != 0x0a )
         *(pt2 + 1) = 0x0a;

      *pt1++ = *pt2;
   }
   _retc( ret );
}
